'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _neon_new = require('./ops/neon_new');

var _neon_new2 = _interopRequireDefault(_neon_new);

var _neon_build = require('./ops/neon_build');

var _neon_build2 = _interopRequireDefault(_neon_build);

var _style = require('./ops/style');

var style = _interopRequireWildcard(_style);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function printUsage() {
  console.log();
  console.log("Usage:");
  console.log();
  console.log("  neon new [@<scope>/]<name> [--rust|-r nightly|stable|default]");
  console.log("    create a new Neon project");
  console.log();
  console.log("  neon build [--rust|-r nightly|stable|default] [--debug|-d]");
  console.log("    rebuild the project");
  console.log();
  console.log("  neon version");
  console.log("    print neon-cli version");
  console.log();
  console.log("  neon help");
  console.log("    print this usage information");
  console.log();
  console.log("neon-cli@" + _package2.default.version + " " + _path2.default.dirname(__dirname));
}

const SUBCOMMANDS = {
  'version': function version() {
    console.log(_package2.default.version);
  },

  'help': function help() {
    printUsage();
  },

  'new': function _new() {
    if (this.args._.length !== 1) {
      printUsage();
      console.log();
      throw new Error(this.args._.length === 0 ? "You must specify a project name." : "Too many arguments.");
    }
    return (0, _neon_new2.default)(this.cwd, this.args._[0], this.args.rust || this.args.r || 'default');
  },

  'build': function build() {
    if (this.args._.length > 0) {
      printUsage();
      console.log();
      throw new Error("Too many arguments.");
    }
    return (0, _neon_build2.default)(this.cwd, this.args.rust || this.args.r || 'default', this.args.debug || this.args.d ? 'debug' : 'release', this.args.node_module_version);
  }
};

class CLI {
  constructor(argv, cwd) {
    this.command = argv[2];
    this.args = (0, _minimist2.default)(argv.slice(3));
    this.cwd = cwd;
  }

  exec() {
    var _this = this;

    return _asyncToGenerator(function* () {
      try {
        if (!_this.command) {
          printUsage();
          throw null;
        }
        if (!SUBCOMMANDS.hasOwnProperty(_this.command)) {
          printUsage();
          console.log();
          throw new Error("'" + _this.command + "' is not a neon command.");
        }
        yield SUBCOMMANDS[_this.command].call(_this);
      } catch (e) {
        if (e) {
          console.log(style.error(e.message));
        }
        throw e;
      }
    })();
  }
}
exports.default = CLI;